use strict;
use Test::More tests => 8;

BEGIN {
    use_ok 'WWW::OpenSearch';
    use_ok 'WWW::OpenSearch::Description';
    use_ok 'WWW::OpenSearch::Response';
    use_ok 'WWW::OpenSearch::Url';
    use_ok 'WWW::OpenSearch::Query';
    use_ok 'WWW::OpenSearch::Image';
    use_ok 'WWW::OpenSearch::Agent';
    use_ok 'WWW::OpenSearch::Request';
}
